package com.example.backend.controller;

import com.example.backend.model.*;
import com.example.backend.service.RadarStatusService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import org.springframework.validation.annotation.Validated;
import com.example.backend.exception.ServiceException;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.util.*;

@RestController
@RequestMapping("/api")
@Validated
public class RadarStatusController {

    private final RadarStatusService radarStatusService;
    private static final Logger logger = LoggerFactory.getLogger(RadarStatusController.class);

    @Autowired
    public RadarStatusController(RadarStatusService radarStatusService) {
        this.radarStatusService = radarStatusService;
    }

    @GetMapping("/platform/getPlatforms")
    public ResponseEntity<List<Platform>> getPlatforms()
            {
        try {
            List list = new ArrayList();
            for(String key : radarStatusService.getPlatforms()){
                Platform p = new Platform();
                p.setPlatformID(key);
                p.setPlatformName("干扰平台-"+key);
                list.add(p);
            }
            return ResponseEntity.ok(list);
        } catch (Exception e) {
            logger.error("获取干扰平台列表", e);
            throw new ServiceException("获取干扰平台列表");
        }
    }

    @GetMapping("/platform/{platformId}/radar-status")
    public ResponseEntity<List<Map<String, Object>>> getRadarStatus(
            @PathVariable @NotBlank String platformId) {
        logger.info("获取平台 {} 的雷达状态", platformId);
        try {
            List<Map<String, Object>> status = radarStatusService.getRadarStatus(platformId);
            return ResponseEntity.ok(status);
        } catch (Exception e) {
            logger.error("获取雷达状态失败", e);
            throw new ServiceException("获取雷达状态失败");
        }
    }

    @PostMapping("/platform/{platformId}/radar-status")
    public ResponseEntity<RadarStatus> updateRadarStatus(
            @PathVariable @NotBlank String platformId,
            @RequestBody @Valid RadarStatus radarStatus) {
        logger.info("更新平台 {} 的雷达状态", platformId);
        try {
            RadarStatus updated = radarStatusService.updateRadarStatus(platformId, radarStatus);
            return ResponseEntity.ok(updated);
        } catch (Exception e) {
            logger.error("更新雷达状态失败", e);
            throw new ServiceException("更新雷达状态失败");
        }
    }

    @PostMapping("/platform/{platformId}/radar-params")
    public ResponseEntity<List<RadarParam>> updateRadarParams(
            @PathVariable @NotBlank String platformId,
            @RequestBody @Valid List<RadarParam> radarParams) {
        logger.info("更新平台 {} 的雷达参数", platformId);
        try {
            List<RadarParam> updated = radarStatusService.updateRadarParams(platformId, radarParams);
            return ResponseEntity.ok(updated);
        } catch (Exception e) {
            logger.error("更新雷达参数失败", e);
            throw new ServiceException("更新雷达参数失败");
        }
    }

    @GetMapping("/platform/{platformId}/radar-params")
    public ResponseEntity<List<RadarParam>> getRadarParams(
            @PathVariable @NotBlank String platformId) {
        logger.info("获取平台 {} 的雷达参数", platformId);
        try {
            List<RadarParam> params = radarStatusService.getRadarParams(platformId);
            return ResponseEntity.ok(params);
        } catch (Exception e) {
            logger.error("获取雷达参数失败", e);
            throw new ServiceException("获取雷达参数失败");
        }
    }

    @GetMapping("/platform/{platformId}/position")
    public ResponseEntity<Position> getPositionById(
            @PathVariable @NotBlank String platformId) {
        try {
            Position p = radarStatusService.getPositionById(platformId);
            if(p==null)p = new Position();
            return ResponseEntity.ok(p);
        } catch (Exception e) {
            logger.error("获取目标位置失败", e);
            throw new ServiceException("获取目标位置失败");
        }
    }

    @GetMapping("/platform/{platformId}/radar-targets")
    public ResponseEntity<List<RadarTarget>> getRadarTargets(
            @PathVariable @NotBlank String platformId) {
        logger.info("获取平台 {} 的目标数据", platformId);
        try {
            List<RadarTarget> targets = radarStatusService.getRadarTargets(platformId);
            return ResponseEntity.ok(targets);
        } catch (Exception e) {
            logger.error("获取目标数据失败", e);
            throw new ServiceException("获取目标数据失败");
        }
    }

    @PostMapping("/platform/{platformId}/radar-targets")
    public ResponseEntity<Map<String, Object>> updateRadarTargets(
            @PathVariable @NotBlank String platformId,
            @RequestBody @Valid List<RadarTarget> targets) {
        logger.info("更新平台 {} 的目标数据", platformId);
        try {
            List<RadarTarget> updated = radarStatusService.updateRadarTargets(platformId, targets);
            Map<String, Object> response = new HashMap<>();
            response.put("targets", updated);
            response.put("queueSize", radarStatusService.getRadarTargetsCount(platformId));
            return ResponseEntity.ok(response);
        } catch (Exception e) {
            logger.error("更新目标数据失败", e);
            throw new ServiceException("更新目标数据失败");
        }
    }

    @DeleteMapping("/platform/{platformId}/radar-targets")
    public ResponseEntity<Void> clearRadarTargets(
            @PathVariable @NotBlank String platformId) {
        logger.info("清除平台 {} 的目标数据", platformId);
        try {
            radarStatusService.clearRadarTargets(platformId);
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            logger.error("清除目标数据失败", e);
            throw new ServiceException("清除目标数据失败");
        }
    }

    @GetMapping("/platform/{platformId}/getMsgStatus")
    public ResponseEntity<MessageStatus> getMsgStatus(
            @PathVariable @NotBlank String platformId) {
        logger.info("获取平台 {} 的目标数据", platformId);
        try {
            MessageStatus m = radarStatusService.getMsgStatusById(platformId);
            return ResponseEntity.ok(m);
        } catch (Exception e) {
            logger.error("获取通信状态失败", e);
            throw new ServiceException("获取通信状态失败");
        }
    }
    //获取合作状态，合作条件 or 非合作条件
    @GetMapping("/platform/{platformId}/getCooperationStatus")
    public ResponseEntity<String> getCooperationStatus(
            @PathVariable @NotBlank String platformId) {
        logger.info("获取平台 {} 的合作状态信息", platformId);
        try {
            String s = radarStatusService.getCooperationStatus(platformId);
            return ResponseEntity.ok(s);
        } catch (Exception e) {
            logger.error("获取合作状态信息失败", e);
            throw new ServiceException("获取合作状态信息失败");
        }
    }

    //获取半实物信息
    @GetMapping("/platform/{platformId}/getIsReal")
    public ResponseEntity<String> getIsReal(
            @PathVariable @NotBlank String platformId) {
        logger.info("获取平台 {} 的半实物信息", platformId);
        try {
            String s = radarStatusService.getIsReal(platformId);
            return ResponseEntity.ok(s);
        } catch (Exception e) {
            logger.error("获取半实物信息失败", e);
            throw new ServiceException("获取半实物信息失败");
        }
    }

    @PostMapping("/platform/{platformId}/setMsgStatus")
    public ResponseEntity<Void> setMsgStatus(
            @PathVariable @NotBlank String platformId,
            @RequestBody @Valid MessageStatus messageStatus) {
        logger.info("更新平台 {} 的通信状态", platformId);
        logger.info(messageStatus.toString());
        try {
            radarStatusService.setMsgStatusById(platformId,messageStatus);
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            logger.error("更新通信状态失败", e);
            throw new ServiceException("更新通信状态失败");
        }
    }

    @GetMapping("/platform/{platformId}/getMsgList")
    public ResponseEntity<List<Message>> getMsgList(
            @PathVariable @NotBlank String platformId) {
        logger.info("获取平台 {} 的消息数据", platformId);
        try {
            List<Message> list = radarStatusService.getMsgListById(platformId);
            return ResponseEntity.ok(list);
        } catch (Exception e) {
            logger.error("获取消息数据失败", e);
            throw new ServiceException("获取消息数据失败");
        }
    }

    @GetMapping("/platform/{platformId}/getOutMsgList")
    public ResponseEntity<List<Message>> getOutMsgList(
            @PathVariable @NotBlank String platformId) {
        logger.info("获取平台 {} 的消息数据", platformId);
        try {
            List<Message> list = radarStatusService.getOutMsgListById(platformId);
            return ResponseEntity.ok(list);
        } catch (Exception e) {
            logger.error("获取消息数据失败", e);
            throw new ServiceException("获取消息数据失败");
        }
    }

    @PostMapping("/platform/{platformId}/sendMsg")
    public ResponseEntity<Void> sendMsg(
            @PathVariable @NotBlank String platformId,
            @RequestBody @Valid Message msg) {
        logger.info("平台 {} 发送消息", platformId);
        try {
            radarStatusService.sendMessageToOther(platformId, msg,msg.getRecivier(),radarStatusService.msgToNumber(msg.getTopic()));
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            logger.error("发送消息失败", e);
            throw new ServiceException("发送消息失败");
        }
    }

    @PostMapping("/platform/{platformId}/sendRadarPulseMsg")
    public ResponseEntity<Void> sendRadarPulseMsg(
            @PathVariable @NotBlank String platformId,
            @RequestBody @Valid radarPulseMsg msg) {
        logger.info("平台 {} 发送消息", platformId);
        try {
            radarStatusService.sendRadarPulseMsg(platformId, msg,msg.getRecivier());
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            logger.error("发送消息失败", e);
            throw new ServiceException("发送消息失败");
        }
    }

    @PostMapping("/platform/{platformId}/sendDisturbMsg")
    public ResponseEntity<Void> sendDisturbMsg(
            @PathVariable @NotBlank String platformId,
            @RequestBody @Valid disturbMsg msg) {
        logger.info("平台 {} 发送消息", platformId);
        try {
            radarStatusService.sendDisturbMsg(platformId, msg,msg.getRecivier());
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            logger.error("发送消息失败", e);
            throw new ServiceException("发送消息失败");
        }
    }

    @PostMapping("/platform/{platformId}/sendCompMsg")
    public ResponseEntity<Void> sendCompMsg(
            @PathVariable @NotBlank String platformId,
            @RequestBody @Valid compMsg msg) {
        logger.info("平台 {} 发送消息", platformId);
        try {
            radarStatusService.sendCompMsg(platformId, msg,msg.getRecivier());
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            logger.error("发送消息失败", e);
            throw new ServiceException("发送消息失败");
        }
    }

    @GetMapping("/platform/{platformId}/getECMStatusByID")
    public ResponseEntity<ecmStatus> getEMCStatusByID(
            @PathVariable @NotBlank String platformId) {
        logger.info("获取平台 {} 的电子攻击状态", platformId);
        try {
            ecmStatus ecmStatus = radarStatusService.getECMStatusByID(platformId);
            return ResponseEntity.ok(ecmStatus);
        } catch (Exception e) {
            logger.error("获取电子攻击状态失败", e);
            throw new ServiceException("获取电子攻击状态失败");
        }
    }

    @GetMapping("/platform/{platformId}/getECMParamsByID")
    public ResponseEntity<List<ecmParams>> getECMParamsByID(
            @PathVariable @NotBlank String platformId) {
        logger.info("获取平台 {} 的干扰列表数据", platformId);
        try {
            List<ecmParams> list = radarStatusService.getECMParamsByID(platformId);
            return ResponseEntity.ok(list);
        } catch (Exception e) {
            logger.error("获取干扰列表数据失败", e);
            throw new ServiceException("获取干扰列表数据失败");
        }
    }

    @PostMapping("/platform/{platformId}/setEcmParam")
    public ResponseEntity<Void> setEcmParam(
            @PathVariable @NotBlank String platformId,
            @RequestBody @Valid ecmParams params) {
        logger.info("平台 {} 发送消息", platformId);
        try {
            radarStatusService.setEcmParam(platformId,params);
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            logger.error("发送消息失败", e);
            throw new ServiceException("发送消息失败");
        }
    }

    @GetMapping("/platform/{platformId}/getTeamMemberByID")
    public ResponseEntity<List<String>> getTeamMemberByID(
            @PathVariable @NotBlank String platformId) {
        logger.info("获取平台 {} 的编队成员数据", platformId);
        try {
            List<String> list = radarStatusService.getTeamMemberByID(platformId);
            return ResponseEntity.ok(list);
        } catch (Exception e) {
            logger.error("获取编队成员数据失败", e);
            throw new ServiceException("获取编队成员数据失败");
        }
    }

    @GetMapping("/platform/{platformId}/getSdrStatusByID")
    public ResponseEntity<sdrStatus> getSdrStatusByID(
            @PathVariable @NotBlank String platformId) {
        logger.info("获取平台 {} 的sdr状态", platformId);
        try {
            sdrStatus s = radarStatusService.getSdrStatusByID(platformId);
            return ResponseEntity.ok(s);
        } catch (Exception e) {
            logger.error("获取sdr状态失败", e);
            throw new ServiceException("获取sdr状态失败");
        }
    }
}
